searchData={"items":[{"type":"module","title":"Xirsys.Sockets.Client","doc":"TCP protocol socket client for TURN server usage.\n\nHandles TCP connections with buffering and provides 5-tuple information\nto callbacks for proper STUN/TURN protocol processing.","ref":"Xirsys.Sockets.Client.html"},{"type":"module","title":"Callback Interface - Xirsys.Sockets.Client","doc":"Your callback module must implement:\n\n    def process_buffer(data, client_ip, client_port, server_ip, server_port)\n\nWhere:\n- `data` - Binary data from TCP stream (may be partial packets)\n- `client_ip` - Client IP address tuple\n- `client_port` - Client port number\n- `server_ip` - Server IP address tuple\n- `server_port` - Server port number\n\nThe function should return `{parsed_packet, remaining_buffer}` where:\n- `parsed_packet` - Complete packet ready for processing (or `nil` if incomplete)\n- `remaining_buffer` - Any remaining unparsed data\n\nThe 5-tuple information is essential for STUN/TURN over TCP (RFC 5389, RFC 5766).","ref":"Xirsys.Sockets.Client.html#module-callback-interface"},{"type":"module","title":"Example - Xirsys.Sockets.Client","doc":"defmodule MyTCPHandler do\n      def process_buffer(data, client_ip, client_port, server_ip, server_port) do\n        case parse_stun_packet(data, client_ip, client_port, server_ip, server_port) do\n          {:ok, packet, rest} -> {packet, rest}\n          {:partial, _} -> {nil, data}  # Wait for more data\n        end\n      end\n\n      def dispatch(conn) do\n        # Process complete STUN/TURN packet\n        handle_packet(conn.message, conn)\n      end\n    end","ref":"Xirsys.Sockets.Client.html#module-example"},{"type":"type","title":"Xirsys.Sockets.Client.callback/0","doc":"","ref":"Xirsys.Sockets.Client.html#t:callback/0"},{"type":"function","title":"Xirsys.Sockets.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Xirsys.Sockets.Client.html#child_spec/1"},{"type":"function","title":"Xirsys.Sockets.Client.create/3","doc":"","ref":"Xirsys.Sockets.Client.html#create/3"},{"type":"function","title":"Xirsys.Sockets.Client.handle_cast/2","doc":"Asynchronous socket response handler","ref":"Xirsys.Sockets.Client.html#handle_cast/2"},{"type":"function","title":"Xirsys.Sockets.Client.handle_info/2","doc":"Message handler to update cache","ref":"Xirsys.Sockets.Client.html#handle_info/2"},{"type":"function","title":"Xirsys.Sockets.Client.init/1","doc":"Callback implementation for  GenServer.init/1 .","ref":"Xirsys.Sockets.Client.html#init/1"},{"type":"function","title":"Xirsys.Sockets.Client.start_link/3","doc":"Standard OTP module startup","ref":"Xirsys.Sockets.Client.html#start_link/3"},{"type":"module","title":"Xirsys.Sockets.Conn","doc":"Socket connection object for TURN","ref":"Xirsys.Sockets.Conn.html"},{"type":"function","title":"Xirsys.Sockets.Conn.halt/1","doc":"Flags a connection object as halted, so it\nshouldn't be processed any further","ref":"Xirsys.Sockets.Conn.html#halt/1"},{"type":"function","title":"Xirsys.Sockets.Conn.response/3","doc":"","ref":"Xirsys.Sockets.Conn.html#response/3"},{"type":"function","title":"Xirsys.Sockets.Conn.send/1","doc":"If a response message has been set, then we must notify the client accordingly","ref":"Xirsys.Sockets.Conn.html#send/1"},{"type":"type","title":"Xirsys.Sockets.Conn.t/0","doc":"","ref":"Xirsys.Sockets.Conn.html#t:t/0"},{"type":"module","title":"Xirsys.Sockets.Listener.SCTP","doc":"Enhanced SCTP protocol socket listener optimized for TURN server usage\nProvides reliable message-oriented communication with multi-streaming support\nIncludes connection management, rate limiting, and robust error handling","ref":"Xirsys.Sockets.Listener.SCTP.html"},{"type":"function","title":"Xirsys.Sockets.Listener.SCTP.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Xirsys.Sockets.Listener.SCTP.html#child_spec/1"},{"type":"function","title":"Xirsys.Sockets.Listener.SCTP.init/1","doc":"Initialises connection with IPv6 address and enhanced options","ref":"Xirsys.Sockets.Listener.SCTP.html#init/1"},{"type":"function","title":"Xirsys.Sockets.Listener.SCTP.start_link/3","doc":"Enhanced OTP module startup with connection tracking","ref":"Xirsys.Sockets.Listener.SCTP.html#start_link/3"},{"type":"module","title":"Xirsys.Sockets.Listener.TCP","doc":"Enhanced TCP protocol socket listener optimized for TURN server usage\nIncludes connection management, rate limiting, and robust error handling","ref":"Xirsys.Sockets.Listener.TCP.html"},{"type":"function","title":"Xirsys.Sockets.Listener.TCP.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Xirsys.Sockets.Listener.TCP.html#child_spec/1"},{"type":"function","title":"Xirsys.Sockets.Listener.TCP.init/1","doc":"Initialises connection with IPv6 address and enhanced options","ref":"Xirsys.Sockets.Listener.TCP.html#init/1"},{"type":"function","title":"Xirsys.Sockets.Listener.TCP.start_link/3","doc":"Enhanced OTP module startup with connection tracking","ref":"Xirsys.Sockets.Listener.TCP.html#start_link/3"},{"type":"function","title":"Xirsys.Sockets.Listener.TCP.start_link/4","doc":"","ref":"Xirsys.Sockets.Listener.TCP.html#start_link/4"},{"type":"module","title":"Xirsys.Sockets.Listener.UDP","doc":"Enhanced UDP protocol socket handler optimized for TURN server usage.\n\nIncludes rate limiting, connection tracking, and robust error handling.\nDesigned specifically for STUN/TURN protocols (RFC 5389, RFC 5766) which\nrequire 5-tuple information for proper operation.","ref":"Xirsys.Sockets.Listener.UDP.html"},{"type":"module","title":"Callback Interface - Xirsys.Sockets.Listener.UDP","doc":"Your callback module must implement:\n\n    def process_buffer(data, client_ip, client_port, server_ip, server_port)\n\nWhere:\n- `data` - Binary packet data received\n- `client_ip` - Client IP address tuple\n- `client_port` - Client port number\n- `server_ip` - Server IP address tuple\n- `server_port` - Server port number\n\nAnd:\n\n    def dispatch(conn)\n\nWhere:\n- `conn` - `%Xirsys.Sockets.Conn{}` struct with parsed message and connection info\n\nThe 5-tuple information during `process_buffer` is essential for:\n- STUN XOR-MAPPED-ADDRESS responses (RFC 5389)\n- TURN allocation and permission management (RFC 5766)\n- Proper WebRTC compatibility","ref":"Xirsys.Sockets.Listener.UDP.html#module-callback-interface"},{"type":"module","title":"Example - Xirsys.Sockets.Listener.UDP","doc":"defmodule MySTUNHandler do\n      def process_buffer(data, client_ip, client_port, server_ip, server_port) do\n        # Parse STUN/TURN packet with full connection context\n        case parse_stun_packet(data, client_ip, client_port, server_ip, server_port) do\n          {:ok, packet} -> {packet, <<>>}\n          {:error, _} -> {nil, <<>>}\n        end\n      end\n\n      def dispatch(conn) do\n        # Handle processed STUN/TURN message\n        send_stun_response(conn)\n      end\n    end","ref":"Xirsys.Sockets.Listener.UDP.html#module-example"},{"type":"function","title":"Xirsys.Sockets.Listener.UDP.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Xirsys.Sockets.Listener.UDP.html#child_spec/1"},{"type":"function","title":"Xirsys.Sockets.Listener.UDP.handle_cast/2","doc":"Enhanced asynchronous socket response handler with rate limiting","ref":"Xirsys.Sockets.Listener.UDP.html#handle_cast/2"},{"type":"function","title":"Xirsys.Sockets.Listener.UDP.handle_info/2","doc":"Enhanced message handler for incoming UDP packets with security checks","ref":"Xirsys.Sockets.Listener.UDP.html#handle_info/2"},{"type":"function","title":"Xirsys.Sockets.Listener.UDP.init/1","doc":"Initialises connection with IPv6 address and enhanced options","ref":"Xirsys.Sockets.Listener.UDP.html#init/1"},{"type":"function","title":"Xirsys.Sockets.Listener.UDP.start_link/3","doc":"Standard OTP module startup with enhanced configuration","ref":"Xirsys.Sockets.Listener.UDP.html#start_link/3"},{"type":"function","title":"Xirsys.Sockets.Listener.UDP.start_link/4","doc":"","ref":"Xirsys.Sockets.Listener.UDP.html#start_link/4"},{"type":"module","title":"Xirsys.Sockets.Response","doc":"TURN connection object","ref":"Xirsys.Sockets.Response.html"},{"type":"type","title":"Xirsys.Sockets.Response.t/0","doc":"","ref":"Xirsys.Sockets.Response.html#t:t/0"},{"type":"module","title":"Xirsys.Sockets.SockSupervisor","doc":"","ref":"Xirsys.Sockets.SockSupervisor.html"},{"type":"function","title":"Xirsys.Sockets.SockSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Xirsys.Sockets.SockSupervisor.html#child_spec/1"},{"type":"function","title":"Xirsys.Sockets.SockSupervisor.init/1","doc":"Callback implementation for  Supervisor.init/1 .","ref":"Xirsys.Sockets.SockSupervisor.html#init/1"},{"type":"function","title":"Xirsys.Sockets.SockSupervisor.start_child/3","doc":"","ref":"Xirsys.Sockets.SockSupervisor.html#start_child/3"},{"type":"function","title":"Xirsys.Sockets.SockSupervisor.start_link/0","doc":"","ref":"Xirsys.Sockets.SockSupervisor.html#start_link/0"},{"type":"function","title":"Xirsys.Sockets.SockSupervisor.terminate_child/1","doc":"","ref":"Xirsys.Sockets.SockSupervisor.html#terminate_child/1"},{"type":"module","title":"Xirsys.Sockets.Socket","doc":"Socket protocol helpers optimized for TURN server usage.\n\nThis module provides a unified interface for handling different socket types\nincluding UDP, TCP, TLS, DTLS, and SCTP protocols. It's specifically optimized\nfor TURN server operations with features like:\n\n- Rate limiting and connection management\n- Enhanced error handling and telemetry\n- Multi-protocol support with consistent API\n- Performance optimizations for media relay\n- Security features and validation","ref":"Xirsys.Sockets.Socket.html"},{"type":"module","title":"Socket Types - Xirsys.Sockets.Socket","doc":"- `:udp` - User Datagram Protocol for connectionless communication\n- `:tcp` - Transmission Control Protocol for reliable connections\n- `:tls` - Transport Layer Security over TCP\n- `:dtls` - Datagram Transport Layer Security over UDP\n- `:sctp` - Stream Control Transmission Protocol with multi-streaming","ref":"Xirsys.Sockets.Socket.html#module-socket-types"},{"type":"module","title":"Examples - Xirsys.Sockets.Socket","doc":"# Open a UDP socket\n    {:ok, socket} = Socket.open_port({127, 0, 0, 1}, :random, [])\n\n    # Send a message\n    :ok = Socket.send(socket, \"Hello\", {192, 168, 1, 1}, 5000)\n\n    # Perform handshake for connection-oriented protocols\n    {:ok, client_socket} = Socket.handshake(tcp_socket)","ref":"Xirsys.Sockets.Socket.html#module-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.check_rate_limit/1","doc":"Enhanced rate limiting check for TURN server protection.\n\nChecks if a client IP is within the rate limit thresholds.\nCreates an ETS table for tracking if it doesn't exist.","ref":"Xirsys.Sockets.Socket.html#check_rate_limit/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.check_rate_limit/1","doc":"- `client_ip` - The client's IP address tuple","ref":"Xirsys.Sockets.Socket.html#check_rate_limit/1-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.check_rate_limit/1","doc":"- `:ok` - Request is within rate limits\n- `{:error, :rate_limited}` - Client has exceeded rate limits","ref":"Xirsys.Sockets.Socket.html#check_rate_limit/1-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.check_rate_limit/1","doc":"iex> Xirsys.Sockets.Socket.check_rate_limit({192, 168, 1, 1})\n    :ok","ref":"Xirsys.Sockets.Socket.html#check_rate_limit/1-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.client_hooks/0","doc":"Returns the client message hooks from configuration.\n\nClient hooks are GenServer processes that receive client messages\nfor processing or logging.","ref":"Xirsys.Sockets.Socket.html#client_hooks/0"},{"type":"function","title":"Xirsys.Sockets.Socket.close/2","doc":"Enhanced socket close with proper logging and cleanup.\n\nCloses sockets with protocol-specific handling and telemetry emission.\nIncludes graceful error handling for unsupported protocols.","ref":"Xirsys.Sockets.Socket.html#close/2"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.close/2","doc":"- `socket` - Socket to close\n- `reason` - Reason for closing (for logging)","ref":"Xirsys.Sockets.Socket.html#close/2-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.close/2","doc":"- `:ok` - Socket closed successfully","ref":"Xirsys.Sockets.Socket.html#close/2-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.close/2","doc":"# Socket.close(socket, \"connection_timeout\")","ref":"Xirsys.Sockets.Socket.html#close/2-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.connection_timeout/0","doc":"Returns the configured connection timeout in milliseconds","ref":"Xirsys.Sockets.Socket.html#connection_timeout/0"},{"type":"function","title":"Xirsys.Sockets.Socket.emit_telemetry/3","doc":"Emit telemetry events for monitoring TURN server performance.\n\nEmits telemetry events with the `:xturn_sockets` prefix for monitoring\nand observability. Gracefully handles cases where telemetry is unavailable.","ref":"Xirsys.Sockets.Socket.html#emit_telemetry/3"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.emit_telemetry/3","doc":"- `event_name` - Name of the event to emit\n- `measurements` - Map of numeric measurements\n- `metadata` - Map of additional metadata","ref":"Xirsys.Sockets.Socket.html#emit_telemetry/3-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.emit_telemetry/3","doc":"- `:ok` - Event emitted successfully","ref":"Xirsys.Sockets.Socket.html#emit_telemetry/3-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.emit_telemetry/3","doc":"iex> Xirsys.Sockets.Socket.emit_telemetry(:connection_opened, %{count: 1}, %{ip: {127, 0, 0, 1}})\n    :ok","ref":"Xirsys.Sockets.Socket.html#emit_telemetry/3-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.get_config/2","doc":"Returns TURN server configuration values.\n\nLooks up configuration values in the following order:\n1. `:xturn_sockets` application config\n2. `:xturn` application config\n3. Default value","ref":"Xirsys.Sockets.Socket.html#get_config/2"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.get_config/2","doc":"- `key` - Configuration key to lookup\n- `default` - Default value if key not found","ref":"Xirsys.Sockets.Socket.html#get_config/2-parameters"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.get_config/2","doc":"iex> Xirsys.Sockets.Socket.get_config(:connection_timeout, 5000)\n    30000","ref":"Xirsys.Sockets.Socket.html#get_config/2-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.getopts/1","doc":"Gets socket options with protocol-aware handling.\n\nRetrieves current socket options based on the socket type.\nDifferent protocols expose different option sets.","ref":"Xirsys.Sockets.Socket.html#getopts/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.getopts/1","doc":"- `socket` - Socket to query options from","ref":"Xirsys.Sockets.Socket.html#getopts/1-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.getopts/1","doc":"- `{:ok, options}` - Successfully retrieved options\n- `{:error, reason}` - Failed to retrieve options","ref":"Xirsys.Sockets.Socket.html#getopts/1-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.getopts/1","doc":"# {:ok, opts} = Socket.getopts(socket)","ref":"Xirsys.Sockets.Socket.html#getopts/1-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.handshake/1","doc":"Enhanced handshake with timeout and better error handling.\n\nPerforms protocol-specific handshakes for connection-oriented sockets.\nIncludes proper timeout handling and telemetry emission.","ref":"Xirsys.Sockets.Socket.html#handshake/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.handshake/1","doc":"- `socket` - The listening socket to accept connections on","ref":"Xirsys.Sockets.Socket.html#handshake/1-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.handshake/1","doc":"- `{:ok, client_socket}` - Successfully accepted connection\n- `{:error, reason}` - Failed to accept or handshake","ref":"Xirsys.Sockets.Socket.html#handshake/1-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.handshake/1","doc":"# TCP handshake example (requires actual socket)\n    # {:ok, client_socket} = Socket.handshake(tcp_listening_socket)","ref":"Xirsys.Sockets.Socket.html#handshake/1-examples"},{"type":"type","title":"Xirsys.Sockets.Socket.ip_address/0","doc":"","ref":"Xirsys.Sockets.Socket.html#t:ip_address/0"},{"type":"function","title":"Xirsys.Sockets.Socket.max_connections_per_ip/0","doc":"Returns the maximum connections allowed per IP address","ref":"Xirsys.Sockets.Socket.html#max_connections_per_ip/0"},{"type":"function","title":"Xirsys.Sockets.Socket.open_port/3","doc":"Opens a new port for UDP TURN transport with enhanced options.\n\nCreates a UDP socket with optimized buffer sizes and TURN-specific\nconfiguration. Supports different port allocation policies.","ref":"Xirsys.Sockets.Socket.html#open_port/3"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.open_port/3","doc":"- `sip` - Server IP address tuple (IPv4)\n- `policy` - Port allocation policy (`:random`, `{:preferred, port}`, `{:range, min, max}`)\n- `opts` - Additional socket options","ref":"Xirsys.Sockets.Socket.html#open_port/3-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.open_port/3","doc":"- `{:ok, socket}` - Successfully opened socket\n- `{:error, reason}` - Failed to open socket","ref":"Xirsys.Sockets.Socket.html#open_port/3-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.open_port/3","doc":"# Open a UDP socket (returns actual socket struct)\n    # Xirsys.Sockets.Socket.open_port({127, 0, 0, 1}, :random, [])","ref":"Xirsys.Sockets.Socket.html#open_port/3-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.peer_hooks/0","doc":"Returns the peer message hooks from configuration.\n\nPeer hooks are GenServer processes that receive peer messages\nfor processing or logging.","ref":"Xirsys.Sockets.Socket.html#peer_hooks/0"},{"type":"function","title":"Xirsys.Sockets.Socket.peername/1","doc":"Returns the peer address and port number for a socket.\n\nGets the remote peer information for connected sockets with protocol-aware handling.","ref":"Xirsys.Sockets.Socket.html#peername/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.peername/1","doc":"- `socket` - Socket to query","ref":"Xirsys.Sockets.Socket.html#peername/1-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.peername/1","doc":"- `{:ok, {ip, port}}` - Successfully retrieved peer address\n- `{:local, path}` - Unix domain socket path\n- `{:unspec, <<>>}` - Unspecified address\n- `{:undefined, term}` - Undefined address\n- `{:error, reason}` - Failed to get peer address","ref":"Xirsys.Sockets.Socket.html#peername/1-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.peername/1","doc":"# {:ok, {ip, port}} = Socket.peername(socket)","ref":"Xirsys.Sockets.Socket.html#peername/1-examples"},{"type":"type","title":"Xirsys.Sockets.Socket.policy/0","doc":"","ref":"Xirsys.Sockets.Socket.html#t:policy/0"},{"type":"function","title":"Xirsys.Sockets.Socket.port/1","doc":"Returns the port number for a UDP socket.\n\nGets the local port number that the socket is bound to.","ref":"Xirsys.Sockets.Socket.html#port/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.port/1","doc":"- `socket` - UDP socket to query","ref":"Xirsys.Sockets.Socket.html#port/1-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.port/1","doc":"- `{:ok, port}` - Successfully retrieved port number\n- `{:error, reason}` - Failed to get port","ref":"Xirsys.Sockets.Socket.html#port/1-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.port/1","doc":"# {:ok, port} = Socket.port(udp_socket)","ref":"Xirsys.Sockets.Socket.html#port/1-examples"},{"type":"type","title":"Xirsys.Sockets.Socket.port_number/0","doc":"","ref":"Xirsys.Sockets.Socket.html#t:port_number/0"},{"type":"function","title":"Xirsys.Sockets.Socket.rate_limit_enabled?/0","doc":"Returns whether rate limiting is enabled","ref":"Xirsys.Sockets.Socket.html#rate_limit_enabled?/0"},{"type":"function","title":"Xirsys.Sockets.Socket.register_socket/1","doc":"Registers a socket with the inet database.\n\nRequired for proper socket management in the Erlang inet system.","ref":"Xirsys.Sockets.Socket.html#register_socket/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.register_socket/1","doc":"- `socket` - Socket to register","ref":"Xirsys.Sockets.Socket.html#register_socket/1-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.register_socket/1","doc":"- `true` - Socket registered successfully\n- `false` - Failed to register socket","ref":"Xirsys.Sockets.Socket.html#register_socket/1-returns"},{"type":"function","title":"Xirsys.Sockets.Socket.send/4","doc":"Sends a message over an open socket with enhanced error handling.\n\nSupports all socket types (UDP, TCP, TLS, DTLS, SCTP) with protocol-specific\noptimizations. Includes telemetry emission for monitoring.","ref":"Xirsys.Sockets.Socket.html#send/4"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.send/4","doc":"- `socket` - The socket to send over\n- `msg` - Binary message to send\n- `ip` - Destination IP (for UDP only)\n- `port` - Destination port (for UDP only)","ref":"Xirsys.Sockets.Socket.html#send/4-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.send/4","doc":"- `:ok` - Message sent successfully\n- `{:error, reason}` - Failed to send message","ref":"Xirsys.Sockets.Socket.html#send/4-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.send/4","doc":"# UDP send\n    # Socket.send(udp_socket, \"hello\", {192, 168, 1, 1}, 5000)\n\n    # TCP send\n    # Socket.send(tcp_socket, \"hello\")","ref":"Xirsys.Sockets.Socket.html#send/4-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.send_to_client_hooks/1","doc":"Sends data to client hooks with error handling.\n\nDispatches messages to configured client hook processes for\nprocessing or logging. Includes error recovery.","ref":"Xirsys.Sockets.Socket.html#send_to_client_hooks/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.send_to_client_hooks/1","doc":"- `data` - The data to send to client hooks","ref":"Xirsys.Sockets.Socket.html#send_to_client_hooks/1-parameters"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.send_to_client_hooks/1","doc":"iex> Xirsys.Sockets.Socket.send_to_client_hooks(%{message: \"hello\"})\n    :ok","ref":"Xirsys.Sockets.Socket.html#send_to_client_hooks/1-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.send_to_peer_hooks/1","doc":"Sends data to peer hooks with error handling.\n\nDispatches messages to configured peer hook processes for\nprocessing or logging. Includes error recovery.","ref":"Xirsys.Sockets.Socket.html#send_to_peer_hooks/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.send_to_peer_hooks/1","doc":"- `data` - The data to send to peer hooks","ref":"Xirsys.Sockets.Socket.html#send_to_peer_hooks/1-parameters"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.send_to_peer_hooks/1","doc":"iex> Xirsys.Sockets.Socket.send_to_peer_hooks(%{message: \"hello\"})\n    :ok","ref":"Xirsys.Sockets.Socket.html#send_to_peer_hooks/1-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.server_ip/0","doc":"Returns the server IP address from configuration.\n\nUsed for packet processing and response generation.","ref":"Xirsys.Sockets.Socket.html#server_ip/0"},{"type":"function","title":"Xirsys.Sockets.Socket.server_local_ip/0","doc":"Returns the server's local IP address from configuration.\n\nUsed for binding sockets to specific network interfaces.","ref":"Xirsys.Sockets.Socket.html#server_local_ip/0"},{"type":"function","title":"Xirsys.Sockets.Socket.set_sockopt/2","doc":"Enhanced socket option setting with better error handling.\n\nTransfers socket options from a listening socket to a client socket.\nIncludes protocol-specific handling and proper error recovery.","ref":"Xirsys.Sockets.Socket.html#set_sockopt/2"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.set_sockopt/2","doc":"- `list_sock` - The listening socket to copy options from\n- `cli_socket` - The client socket to apply options to","ref":"Xirsys.Sockets.Socket.html#set_sockopt/2-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.set_sockopt/2","doc":"- `:ok` - Options transferred successfully\n- `{:error, reason}` - Failed to transfer options","ref":"Xirsys.Sockets.Socket.html#set_sockopt/2-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.set_sockopt/2","doc":"# Socket.set_sockopt(listening_socket, client_socket)","ref":"Xirsys.Sockets.Socket.html#set_sockopt/2-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.setopts/2","doc":"Sets one or more options for a socket.\n\nConfigures socket options with protocol-aware handling.\nSupports both raw ports and Socket structs.","ref":"Xirsys.Sockets.Socket.html#setopts/2"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.setopts/2","doc":"- `socket` - Socket or port to configure\n- `opts` - List of socket options to set","ref":"Xirsys.Sockets.Socket.html#setopts/2-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.setopts/2","doc":"- `:ok` - Options set successfully\n- `{:error, reason}` - Failed to set options","ref":"Xirsys.Sockets.Socket.html#setopts/2-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.setopts/2","doc":"# Socket.setopts(socket, [{:active, :once}, :binary])","ref":"Xirsys.Sockets.Socket.html#setopts/2-examples"},{"type":"type","title":"Xirsys.Sockets.Socket.socket_options/0","doc":"","ref":"Xirsys.Sockets.Socket.html#t:socket_options/0"},{"type":"type","title":"Xirsys.Sockets.Socket.socket_type/0","doc":"","ref":"Xirsys.Sockets.Socket.html#t:socket_type/0"},{"type":"function","title":"Xirsys.Sockets.Socket.sockname/1","doc":"Returns the local address and port number for a socket.\n\nGets the local binding information for the socket with protocol-aware handling.","ref":"Xirsys.Sockets.Socket.html#sockname/1"},{"type":"function","title":"Parameters - Xirsys.Sockets.Socket.sockname/1","doc":"- `socket` - Socket to query","ref":"Xirsys.Sockets.Socket.html#sockname/1-parameters"},{"type":"function","title":"Returns - Xirsys.Sockets.Socket.sockname/1","doc":"- `{:ok, {ip, port}}` - Successfully retrieved local address\n- `{:local, path}` - Unix domain socket path\n- `{:unspec, <<>>}` - Unspecified address\n- `{:undefined, term}` - Undefined address\n- `{:error, reason}` - Failed to get address","ref":"Xirsys.Sockets.Socket.html#sockname/1-returns"},{"type":"function","title":"Examples - Xirsys.Sockets.Socket.sockname/1","doc":"# {:ok, {ip, port}} = Socket.sockname(socket)","ref":"Xirsys.Sockets.Socket.html#sockname/1-examples"},{"type":"function","title":"Xirsys.Sockets.Socket.ssl_handshake_timeout/0","doc":"Returns the configured SSL handshake timeout in milliseconds","ref":"Xirsys.Sockets.Socket.html#ssl_handshake_timeout/0"},{"type":"type","title":"Xirsys.Sockets.Socket.t/0","doc":"","ref":"Xirsys.Sockets.Socket.html#t:t/0"},{"type":"module","title":"Xirsys.Sockets.Telemetry","doc":"Telemetry handlers for monitoring TURN server socket performance\n\nThis module provides comprehensive monitoring for:\n- Connection metrics\n- Data transfer rates\n- Error rates\n- Rate limiting events\n- SSL/TLS handshake performance","ref":"Xirsys.Sockets.Telemetry.html"},{"type":"function","title":"Xirsys.Sockets.Telemetry.attach_handlers/0","doc":"Attach telemetry handlers for socket monitoring","ref":"Xirsys.Sockets.Telemetry.html#attach_handlers/0"},{"type":"function","title":"Xirsys.Sockets.Telemetry.detach_handlers/0","doc":"Detach all telemetry handlers","ref":"Xirsys.Sockets.Telemetry.html#detach_handlers/0"},{"type":"function","title":"Xirsys.Sockets.Telemetry.get_metrics/0","doc":"Get current socket metrics","ref":"Xirsys.Sockets.Telemetry.html#get_metrics/0"},{"type":"function","title":"Xirsys.Sockets.Telemetry.health_status/0","doc":"Get health status based on error rates","ref":"Xirsys.Sockets.Telemetry.html#health_status/0"},{"type":"function","title":"Xirsys.Sockets.Telemetry.reset_counters/0","doc":"Reset all counters (useful for testing)","ref":"Xirsys.Sockets.Telemetry.html#reset_counters/0"},{"type":"extras","title":"XTurn Sockets","doc":"# XTurn Sockets\n\nA high-performance, production-ready Elixir socket library optimized for TURN server usage. Provides unified support for multiple transport protocols with advanced features like rate limiting, connection management, and comprehensive telemetry.","ref":"readme.html"},{"type":"extras","title":"🚀 Features - XTurn Sockets","doc":"- **Multi-Protocol Support**: UDP, TCP, TLS, DTLS, and SCTP protocols\n- **Production Optimized**: Built specifically for TURN server requirements\n- **Rate Limiting**: Per-IP rate limiting with configurable thresholds\n- **Connection Management**: Automatic connection tracking and cleanup\n- **Telemetry Integration**: Comprehensive metrics and monitoring\n- **Security Features**: SSL/TLS support with configurable certificates\n- **Error Handling**: Robust error handling with graceful degradation\n- **Performance Tuned**: Optimized buffer sizes and connection pooling\n- **IPv4/IPv6 Support**: Full dual-stack networking support","ref":"readme.html#features"},{"type":"extras","title":"📦 Installation - XTurn Sockets","doc":"Add `xturn_sockets` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:xturn_sockets, \"~> 1.1.0\"},\n    {:telemetry, \"~> 1.0\"}  # For telemetry features\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"🔧 Configuration - XTurn Sockets","doc":"Configure the library in your `config/config.exs`:\n\n```elixir\nconfig :xturn_sockets,\n  # Connection Management\n  connection_timeout: 30_000,           # 30 seconds\n  max_udp_connections: 10_000,          # Max UDP connections\n  max_tcp_connections: 1_000,           # Max TCP connections\n  max_sctp_connections: 1_000,          # Max SCTP connections\n\n  # Rate Limiting\n  rate_limit_enabled: true,\n  rate_limit_window: 60_000,            # 1 minute window (1000 requests per window is hardcoded)\n\n  # Buffer Configuration\n  buffer_size: 262_144,                 # 256KB buffers\n\n  # SSL/TLS Configuration\n  ssl_handshake_timeout: 10_000,        # 10 seconds\n\n  # SCTP Configuration\n  sctp_nodelay: true,\n  sctp_autoclose: 0,\n  sctp_maxseg: 1400,\n\n  # Telemetry\n  telemetry_enabled: true\n\n# Certificate configuration for TLS/DTLS\nconfig :certs,\n  certfile: \"path/to/cert.pem\",\n  keyfile: \"path/to/key.pem\",\n  cacertfile: \"path/to/ca.pem\"\n```","ref":"readme.html#configuration"},{"type":"extras","title":"📖 Usage Examples - XTurn Sockets","doc":"","ref":"readme.html#usage-examples"},{"type":"extras","title":"Basic UDP Socket - XTurn Sockets","doc":"```elixir\nalias Xirsys.Sockets.Socket\n\n# Open a UDP socket\n{:ok, socket} = Socket.open_port({127, 0, 0, 1}, :random, [])\n\n# Send a message\n:ok = Socket.send(socket, \"Hello UDP\", {192, 168, 1, 100}, 5000)\n\n# Get socket information\n{:ok, {ip, port}} = Socket.sockname(socket)\n\n# Close the socket\n:ok = Socket.close(socket)\n```","ref":"readme.html#basic-udp-socket"},{"type":"extras","title":"UDP Listener with Callback - XTurn Sockets","doc":"```elixir\ndefmodule MyUDPHandler do\n  def process_buffer(data, client_ip, client_port, server_ip, server_port) do\n    # Process incoming data with full 5-tuple information for STUN/TURN protocols\n    # This is essential for RFC 5389 (STUN) and RFC 5766 (TURN) compliance\n    {data, <<>>}  # Return {processed_data, remaining_buffer}\n  end\n\n  def dispatch(conn) do\n    # Handle the processed connection\n    IO.puts \"Received: #{inspect(conn.message)}\"\n    IO.puts \"From: #{inspect(conn.client_ip)}:#{conn.client_port}\"\n  end\nend\n\n# Start UDP listener\n{:ok, pid} = Xirsys.Sockets.Listener.UDP.start_link(\n  MyUDPHandler,\n  {0, 0, 0, 0},  # Bind to all interfaces\n  8080           # Port\n)\n```","ref":"readme.html#udp-listener-with-callback"},{"type":"extras","title":"TCP Listener - XTurn Sockets","doc":"```elixir\ndefmodule MyTCPHandler do\n  def process_buffer(data, client_ip, client_port, server_ip, server_port) do\n    # Process TCP data with 5-tuple information\n    {data, <<>>}  # Return {processed_data, remaining_buffer}\n  end\n\n  def dispatch(conn) do\n    # Handle the processed connection\n    Socket.send(conn.client_socket, \"Echo: #{conn.message}\")\n    :ok\n  end\nend\n\n# Start TCP listener\n{:ok, pid} = Xirsys.Sockets.Listener.TCP.start_link(\n  MyTCPHandler,\n  {0, 0, 0, 0},\n  8081\n)\n```","ref":"readme.html#tcp-listener"},{"type":"extras","title":"DTLS (Secure UDP) Listener - XTurn Sockets","doc":"```elixir\n# Start DTLS listener (requires certificate configuration)\n{:ok, pid} = Xirsys.Sockets.Listener.UDP.start_link(\n  MyUDPHandler,\n  {0, 0, 0, 0},\n  8443,\n  {:ssl, true}  # Enable DTLS\n)\n```","ref":"readme.html#dtls-secure-udp-listener"},{"type":"extras","title":"SCTP Multi-Stream Support - XTurn Sockets","doc":"```elixir\ndefmodule MySCTPHandler do\n  def process_buffer(data, client_ip, client_port, server_ip, server_port) do\n    # Process SCTP data with 5-tuple information\n    {data, <<>>}  # Return {processed_data, remaining_buffer}\n  end\n\n  def dispatch(conn) do\n    # Handle the processed SCTP connection with metadata\n    stream_id = conn.metadata[:stream_id] || 0\n    ppid = conn.metadata[:ppid] || 0\n\n    IO.puts \"SCTP message on stream #{stream_id}, PPID #{ppid}\"\n    IO.puts \"Data: #{inspect(conn.message)}\"\n\n    # Echo back on the same stream\n    Socket.send(conn.client_socket, \"Echo: #{conn.message}\")\n    :ok\n  end\nend\n\n# Start SCTP listener\n{:ok, pid} = Xirsys.Sockets.Listener.SCTP.start_link(\n  MySCTPHandler,\n  {0, 0, 0, 0},\n  8082\n)\n```","ref":"readme.html#sctp-multi-stream-support"},{"type":"extras","title":"Advanced Socket Operations - XTurn Sockets","doc":"```elixir\n# Create socket with specific options\n{:ok, socket} = Socket.open_port(\n  {127, 0, 0, 1},\n  {:range, 50000, 50100},  # Port range\n  [\n    {:reuseaddr, true},\n    {:buffer, 65536}\n  ]\n)\n\n# Set socket options\n:ok = Socket.setopts(socket, [{:active, :once}, :binary])\n\n# Get socket options\n{:ok, opts} = Socket.getopts(socket)\n\n# Check rate limiting\ncase Socket.check_rate_limit({192, 168, 1, 100}) do\n  :ok ->\n    # Process request\n    :ok\n  {:error, :rate_limited} ->\n    # Reject request\n    {:error, :too_many_requests}\nend\n```","ref":"readme.html#advanced-socket-operations"},{"type":"extras","title":"📊 Telemetry and Monitoring - XTurn Sockets","doc":"The library provides comprehensive telemetry events for monitoring:\n\n```elixir\n# Attach telemetry handlers\nXirsys.Sockets.Telemetry.attach_handlers()\n\n# Get current metrics\nmetrics = Xirsys.Sockets.Telemetry.get_metrics()\n\n# Example metrics structure:\n%{\n  total_connections: 1500,\n  active_udp_connections: 800,\n  active_tcp_connections: 500,\n  active_sctp_connections: 200,\n  bytes_sent: 1_048_576,\n  bytes_received: 2_097_152,\n  rate_limit_hits: 45,\n  ssl_handshake_successes: 150,\n  ssl_handshake_errors: 5,\n  send_errors: 12,\n  last_updated: 1640995200\n}\n\n# Check health status\nhealth = Xirsys.Sockets.Telemetry.health_status()\n# Returns: :healthy | :degraded | :unhealthy | :under_attack\n```","ref":"readme.html#telemetry-and-monitoring"},{"type":"extras","title":"Custom Telemetry Handlers - XTurn Sockets","doc":"```elixir\n# Attach custom handler for connection events\n:telemetry.attach(\n  \"my-connection-handler\",\n  [:xturn_sockets, :connection_accepted],\n  fn _name, measurements, metadata, _config ->\n    protocol = measurements[:protocol]\n    Logger.info(\"New #{protocol} connection from #{inspect(metadata[:ip])}\")\n  end,\n  %{}\n)\n```","ref":"readme.html#custom-telemetry-handlers"},{"type":"extras","title":"🔧 Callback Interface - XTurn Sockets","doc":"### **IMPORTANT: 5-Tuple Interface for STUN/TURN**\n\nAll callback modules **must** implement the following interface:\n\n```elixir\ndef process_buffer(data, client_ip, client_port, server_ip, server_port)\n```\n\nThis interface provides the complete 5-tuple information required by:\n\n- **RFC 5389 (STUN)**: For XOR-MAPPED-ADDRESS responses\n- **RFC 5766 (TURN)**: For allocation and permission management\n- **WebRTC compatibility**: Proper NAT traversal responses\n\n### **Why 5-Tuple is Critical**\n\nSTUN/TURN protocols require the complete connection context:\n\n```elixir\ndefmodule MySTUNHandler do\n  def process_buffer(data, client_ip, client_port, server_ip, server_port) do\n    case STUNPacket.parse(data) do\n      {:binding_request, transaction_id} ->\n        # Build XOR-MAPPED-ADDRESS using client IP/port\n        response = STUNPacket.binding_response(\n          transaction_id,\n          client_ip,\n          client_port,\n          server_ip,  # Required for XOR calculation\n          server_port\n        )\n        {response, <<>>}\n\n      _ -> {nil, <<>>}\n    end\n  end\n\n  def dispatch(conn) do\n    # Send response back to client\n    Socket.send(conn.client_socket, conn.message, conn.client_ip, conn.client_port)\n  end\nend\n```\n\n### **Return Format**\n\nThe `process_buffer/5` function must return:\n\n- `{packet, remaining_buffer}` - Complete packet with any remaining data\n- `{nil, buffer}` - Incomplete packet, wait for more data","ref":"readme.html#callback-interface"},{"type":"extras","title":"🏗️ Architecture - XTurn Sockets","doc":"","ref":"readme.html#architecture"},{"type":"extras","title":"Socket Types - XTurn Sockets","doc":"- **UDP**: Connectionless, high-performance datagram transport\n- **TCP**: Reliable, connection-oriented stream transport\n- **TLS**: Secure TCP with SSL/TLS encryption\n- **DTLS**: Secure UDP with datagram TLS encryption\n- **SCTP**: Message-oriented transport with multi-streaming","ref":"readme.html#socket-types"},{"type":"extras","title":"Components - XTurn Sockets","doc":"- `Socket`: Core socket abstraction and operations\n- `Listener.UDP`: UDP packet listener with DTLS support\n- `Listener.TCP`: TCP connection listener with TLS support\n- `Listener.SCTP`: SCTP association listener with multi-streaming\n- `Telemetry`: Metrics collection and health monitoring\n- `Client`: Client connection management\n- `Conn`: Connection state and metadata","ref":"readme.html#components"},{"type":"extras","title":"🔐 Security Features - XTurn Sockets","doc":"","ref":"readme.html#security-features"},{"type":"extras","title":"Rate Limiting - XTurn Sockets","doc":"```elixir\n# Per-IP rate limiting (1000 requests/minute by default)\ncase Socket.check_rate_limit(client_ip) do\n  :ok ->\n    # Within limits\n    process_request()\n  {:error, :rate_limited} ->\n    # Rate limited\n    send_error_response()\nend\n```","ref":"readme.html#rate-limiting"},{"type":"extras","title":"SSL/TLS Configuration - XTurn Sockets","doc":"```elixir\n# Configure certificates\nconfig :certs,\n  certfile: \"/path/to/server.crt\",\n  keyfile: \"/path/to/server.key\",\n  cacertfile: \"/path/to/ca.crt\",\n  # Additional SSL options\n  versions: [:\"tlsv1.2\", :\"tlsv1.3\"],\n  ciphers: [:ecdhe_ecdsa_aes_256_gcm_sha384, :ecdhe_rsa_aes_256_gcm_sha384],\n  secure_renegotiate: true,\n  reuse_sessions: true\n```","ref":"readme.html#ssl-tls-configuration"},{"type":"extras","title":"Connection Limits - XTurn Sockets","doc":"```elixir\n# Automatic connection limiting per protocol\nconfig :xturn_sockets,\n  max_udp_connections: 10_000,\n  max_tcp_connections: 1_000,\n  max_sctp_connections: 1_000,\n  max_connections_per_ip: 100\n```","ref":"readme.html#connection-limits"},{"type":"extras","title":"🧪 Testing - XTurn Sockets","doc":"Run the test suite:\n\n```bash\n# Run all tests\nmix test\n\n# Run tests for specific protocols\nmix test --only udp\nmix test --only tcp\nmix test --only tls\nmix test --only dtls\nmix test --only sctp\n\n# Run with coverage\nmix test --cover\n```","ref":"readme.html#testing"},{"type":"extras","title":"📈 Performance Tuning - XTurn Sockets","doc":"","ref":"readme.html#performance-tuning"},{"type":"extras","title":"Buffer Sizes - XTurn Sockets","doc":"```elixir\nconfig :xturn_sockets,\n  buffer_size: 262_144,        # 256KB (default)\n  # For high-throughput applications:\n  buffer_size: 1_048_576       # 1MB\n```","ref":"readme.html#buffer-sizes"},{"type":"extras","title":"Connection Pooling - XTurn Sockets","doc":"```elixir\nconfig :xturn_sockets,\n  connection_cleanup_enabled: true,\n  connection_cleanup_interval: 300_000,  # 5 minutes\n  connection_timeout: 30_000             # 30 seconds\n```","ref":"readme.html#connection-pooling"},{"type":"extras","title":"SCTP Optimization - XTurn Sockets","doc":"```elixir\nconfig :xturn_sockets,\n  sctp_nodelay: true,           # Disable Nagle algorithm\n  sctp_autoclose: 0,            # Disable auto-close\n  sctp_maxseg: 1400,            # Optimal segment size\n  sctp_streams: %{\n    num_ostreams: 10,           # Outbound streams\n    max_instreams: 10,          # Inbound streams\n    max_attempts: 4,            # Connection attempts\n    max_init_timeo: 30_000      # Init timeout\n  }\n```","ref":"readme.html#sctp-optimization"},{"type":"extras","title":"🐛 Troubleshooting - XTurn Sockets","doc":"","ref":"readme.html#troubleshooting"},{"type":"extras","title":"Common Issues - XTurn Sockets","doc":"1. **SCTP Not Available**\n\n   ```\n   {:error, :sctp_not_supported}\n   ```\n\n   Solution: Ensure SCTP support is compiled into your Erlang/OTP installation.\n\n2. **Certificate Errors**\n\n   ```\n   {:error, :no_certificates_configured}\n   ```\n\n   Solution: Configure certificates in the `:certs` application environment.\n\n3. **Rate Limiting False Positives**\n   ```\n   {:error, :rate_limited}\n   ```\n   Solution: Adjust rate limiting configuration or disable for testing.","ref":"readme.html#common-issues"},{"type":"extras","title":"Debug Mode - XTurn Sockets","doc":"```elixir\n# Enable debug logging\nconfig :logger, level: :debug\n\n# Enable socket statistics\n{:ok, pid} = Xirsys.Sockets.Listener.UDP.start_link(\n  MyHandler,\n  {0, 0, 0, 0},\n  8080,\n  false,\n  debug: [:statistics]\n)\n```","ref":"readme.html#debug-mode"},{"type":"extras","title":"🤝 Contributing - XTurn Sockets","doc":"1. Fork the repository\n2. Create a feature branch (`git checkout -b feature/amazing-feature`)\n3. Make your changes\n4. Add tests for new functionality\n5. Ensure all tests pass (`mix test`)\n6. Commit your changes (`git commit -m 'Add amazing feature'`)\n7. Push to the branch (`git push origin feature/amazing-feature`)\n8. Open a Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"📜 License - XTurn Sockets","doc":"Copyright (c) 2013 - 2020 Xirsys LLC\n\nLicensed under the Apache License, Version 2.0. See [LICENSE](LICENSE.md) for details.","ref":"readme.html#license"},{"type":"extras","title":"💝 Donations - XTurn Sockets","doc":"Developing libraries takes time (mostly personal), so donations are welcome. Donations can be sent via PayPal:\n\n![Donation QR Code](./docs/assets/images/qrcode.png)","ref":"readme.html#donations"},{"type":"extras","title":"📞 Contact - XTurn Sockets","doc":"For questions, suggestions, or support:\n\n- Email: experts@xirsys.com\n- Issues: [GitHub Issues](https://github.com/xirsys/xturn-sockets/issues)","ref":"readme.html#contact"},{"type":"extras","title":"🔗 Related Projects - XTurn Sockets","doc":"- [XTurn](https://github.com/xirsys/xturn) - TURN/STUN server implementation\n- [Xirsys Cloud](https://xirsys.com) - Managed TURN/STUN service","ref":"readme.html#related-projects"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}